"""Configuration management for EasyTransfer.

The actual server configuration is in etransfer.server.config.ServerSettings.
This module is kept for the public API constants and utility functions.
"""

from etransfer.common.constants import CACHE_DIR_NAME, DEFAULT_CHUNK_SIZE, DEFAULT_SERVER_PORT
